//	TorusGames-3D-Wall.vs
//
//	© 2013 by Jeff Weeks
//	See TermsOfUse.txt

in mat4			atrWallIntoWorld;
uniform mat4	uniProjection;		//	maps world space to clip space

uniform vec3	uniFogDistances;	//	(z_begin, z_end, 1.0/(z_end - z_begin))
uniform vec2	uniAperture;		//	(a, 1 - a);  (0,1) = fully closed, (1,0) = fully open

in vec3			atrOuterPosition;	//	(x,y,z) of outer vertex
in vec4			atrColor;			//	premultiplied alpha
in vec2			atrOuterTexCoords;
in vec2			atrWeights;			//	(outer vertex weight, inner vertex weight)
in vec2			atrTexCoordOffset;	//	(∆s, ∆t) to let the texture move with the game cell
									//		rather than staying fixed on the frame cell

out vec2		varTextureCoordinates;
out vec4		varDiffuseColor;	//	premultiplied alpha

const vec3		cntWallCenterPosition	= vec3(0.0, 0.0, 0.0);
const vec2		cntWallCenterTexCoords	= vec2(0.5, 0.5);

const vec4		cntWallNormal			= vec4(0.0, 0.0, -1.0, 0.0);
const vec3		cntLightPosition		= vec3(0.0, 0.0, -1.0);

void main()
{
	vec3	tmpInnerPosition,		//	in wall space
			tmpWeightedPosition;	//	in wall space
	vec4	tmpWorldPosition;		//	in world space
	vec2	tmpInnerTexCoords,
			tmpWeightedTexCoords;
	float	tmpFogFactor;
	vec3	tmpLightDirection,
			tmpNormal;
	float	tmpDiffuseFactor,
			tmpBrightness;
	
	//	position
	tmpInnerPosition	= uniAperture[0] * atrOuterPosition
						+ uniAperture[1] * cntWallCenterPosition;
	tmpWeightedPosition	= atrWeights[0] * atrOuterPosition
						+ atrWeights[1] * tmpInnerPosition;
	tmpWorldPosition	= atrWallIntoWorld * vec4(tmpWeightedPosition, 1.0);
	gl_Position			= uniProjection * tmpWorldPosition;

	//	texture coordinates
	tmpInnerTexCoords		= uniAperture[0] * atrOuterTexCoords
							+ uniAperture[1] * cntWallCenterTexCoords;
	tmpWeightedTexCoords	= atrWeights[0] * atrOuterTexCoords
							+ atrWeights[1] * tmpInnerTexCoords;
	varTextureCoordinates	= tmpWeightedTexCoords + atrTexCoordOffset;
	
	//	fog
	//
	//		tmpFogFactor = (z_end - z) / (z_end - z_begin)
	//
	//	clamped to [0,1].  (Clamping is surely unnecessary here.)
	tmpFogFactor = clamp(
					(uniFogDistances[1] - tmpWorldPosition[2]) * uniFogDistances[2],
					0.0,
					1.0);

	//	point light
	tmpLightDirection	= normalize(cntLightPosition - vec3(tmpWorldPosition));
	tmpNormal			= vec3(atrWallIntoWorld * cntWallNormal);
	tmpDiffuseFactor	= 0.5 + 0.5*dot(tmpLightDirection, tmpNormal);

	//	final color
	tmpBrightness	= tmpFogFactor * tmpDiffuseFactor;
	varDiffuseColor	= vec4(tmpBrightness, tmpBrightness, tmpBrightness, 1.0) * atrColor;
}
